#!/usr/bin/env python3
import logging
import os
import signal

from mobilevids import __VERSION__
from .options import options_parser
from .define import DOWNLOAD_DIRECTORY
from .network import session_init, get_creds
from .downloader import Downloader


def main():
	args = options_parser()

	logging.debug(f'Version: {__VERSION__}')

	if not os.path.exists(DOWNLOAD_DIRECTORY):
		logging.debug(f'Creating Directory {DOWNLOAD_DIRECTORY}')
		os.makedirs(DOWNLOAD_DIRECTORY, exist_ok=True)


	session = session_init()
	auth_token, user_id = get_creds(session, args.username, args.password)

	downloader = Downloader(session, auth_token, user_id, args.ascii, args.info)
	signal.signal(signal.SIGINT, downloader.signal_handler)


	if args.search:
		downloader.search(args.search)
	elif args.movie:
		downloader.get_movie_by_id(args.movie)
	elif args.tv:
		if args.episode and args.season:
			downloader.get_single_episode(args.tv, args.season, args.episode, DOWNLOAD_DIRECTORY)
		elif args.season:
			downloader.get_show_by_id(args.tv, args.season)
		else:
			downloader.get_show_by_id(args.tv)
	else:
		downloader.search()



import os

from mobilevids import __PKGNAME__

def normalize_path(path):
		home_dir = os.path.expanduser('~')
		if os.name == 'nt':
			package_dir = os.path.join(home_dir, "AppData", "Local", __PKGNAME__)
		else:
			package_dir = home_dir
    
		return os.path.join(package_dir, path)

LEGACY_URL = 'https://mobilevids.org/legacy'
BASE_URL = 'https://mobilevids.org/'
LOGIN_URL = BASE_URL + 'webapi/user/login.php'
SEARCH_URL = BASE_URL + 'webapi/videos/search.php?&p=1&user_id={}&token={}&query={}'
GET_VIDEO_URL = BASE_URL + 'webapi/videos/get_video.php?user_id={}&token={}&id={}'
GET_SEASON_URL = BASE_URL + 'webapi/videos/get_season.php?user_id={}&token={}&show_id={}'
GET_SINGLE_EPISODE_URL = BASE_URL + 'webapi/videos/get_single_episode.php?user_id={}&token={}&show_id={}&season={}&episode={}'
LOGIN_PAYLOAD = 'data=%7B%22Name%22%3A%22{username}%22%2C%22Password%22%3A%22{password}%22%7D'
DOWNLOAD_DIRECTORY = normalize_path('downloads/')
QUALITIES = ['src_vip_hd_1080p', 'src_vip_hd', 'src_vip_sd', 'src_free_sd']
HEADERS = {'POST': '/webapi/user/login.php HTTP/1.1',
						'Host': 'mobilevids.org',
						'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:94.0) Gecko/20100101 Firefox/94.0',
						'Accept': '*/*',
						'Accept-Language': 'en-US,en;q=0.5',
						'Accept-Encoding': 'gzip, deflate, br',
						'sec-ch-ua' : '".Not/A)Brand";v="99", "Google Chrome";v="103", "Chromium";v="103"',
						'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
						'Cookie': 'menuNodes=[{"name":"Movies","icon":"mdi-movie","children":[{"name":"Browse","path":"/movies","icon":"mdi-view-list"},{"name":"Popular","path":"/top_movies","icon":"mdi-movie-filter"}]},{"name":"TV Shows","icon":"mdi-remote-tv","children":[{"name":"Browse ","path":"/tvshows","icon":"mdi-view-list"},{"name":"Popular ","path":"/top_shows","icon":"mdi-monitor-star"},{"name":"Calendar","path":"/show_calendar","icon":"mdi-calendar"}]}]',
						'X-Requested-With': 'XMLHttpRequest',
						'Content-Length': '77',
						'Origin': 'https://mobilevids.org',
						'Connection': 'keep-alive',
						'Referer': 'https://mobilevids.org/legacy/',
						'Sec-Fetch-Dest': 'empty',
						'Sec-Fetch-Mode': 'cors',
						'Sec-Fetch-Site': 'same-origin',
						'sec-ch-ua-platform' : '"Windows"'
						}
AUTH_TOKEN_CACHE = normalize_path('.mvdl_auth_token')
NETRC_FILE_PATH  = normalize_path('.netrc')



import os
import logging
import html

from .define import DOWNLOAD_DIRECTORY, QUALITIES, SEARCH_URL, GET_VIDEO_URL, GET_SEASON_URL, GET_SINGLE_EPISODE_URL
from .imagetoascii import image_to_ascii
from .network import get_json, dl_wrapper


class Downloader:
	"""
    Class for downloading movies and TV shows from the streaming service.

    Attributes:
        session (requests.Session): The session object used for making HTTP requests.
        auth_token (str): Authentication token used for accessing the streaming service.
        user_id (str): User ID associated with the streaming service account.
        ascii (bool): Whether to display movie/TV show poster thumbnails in ASCII art.
        info (bool): Whether to display information about the movie/TV show.
        download_dir (str): Directory where downloaded files will be stored.
        dl_obj (dl_wrapper): Download object for handling file downloads.

    Methods:
        get_quality(info: dict) -> str:
            Returns the best quality available for the given movie or TV show.

        search(search_query: str = ''):
            Searches for a movie or TV show with the given search query and prompts the user to select one to download.

        get_movie_by_id(movie_id: str):
            Downloads the movie with the given ID.

        get_show_by_id(show_id: str, season_chosen=None):
            Downloads the TV show with the given ID and prompts the user to select a season to download.

        get_single_episode(show_id: str, season: str, episode: str, path: str):
            Downloads a single episode of the given TV show.

        signal_handler(sig, frame):
            Handles keyboard interrupts during file downloads and removes incomplete downloads.
  	"""
	def __init__(self, session, auth_token, user_id, ascii=False, info=False) -> None:
		self.session = session
		self.auth_token = auth_token
		self.user_id = user_id
		self.ascii = ascii
		self.info = info
		self.download_dir = DOWNLOAD_DIRECTORY
		self.dl_obj = None


	def get_quality(self, info): 
		"""
    Returns the best quality available for the given movie or TV show.

    Args:
        info (dict): Dictionary containing information about the movie or TV show.

    Returns:
        str: The quality (e.g. "1080p") of the video with the highest available quality.
        
    Raises:
        Exception: If no video is found for the given URL.
    	"""
		for quality in QUALITIES:
			if quality in info and info[quality] != '':
				return info[quality]
		raise Exception(
			'No video found for the given URL'
		)


	def search(self, search_query:str = ''):
		"""
    Searches for a movie or TV show with the given search query and prompts the user to select one to download.

    Args:
        search_query (str): The search query to use. If not provided, the user will be prompted to enter one.
    """
		if not search_query:
			search_query = input('[?] Search for something: ').lower()
		response = get_json(self.session, SEARCH_URL.format(self.user_id, self.auth_token, search_query))

		if response['items'] == None:
				logging.error(f'[!] No results found for "{search_query}" - exiting!')
				exit()

		if len(response['items']) == 1:
			logging.info("[!] Only one result found - downloading it!")
			first_id = response['items'][0]
			self.get_movie_by_id(first_id['id']) if first_id['cat_id'] == 1 else self.get_show_by_id(first_id['id'])
			exit()

		logging.info("Search results: ")
		for counter, i in enumerate(response['items']):
			logging.info(f'{str(counter + 1)}) '
									 f'ID: {str(i["id"])} '
									 f'Name: {html.unescape(i["title"])} {"(Movie)" if i["cat_id"] == 1 else "(TV)"}'
									 )

			if self.ascii:
				image_to_ascii(i['poster_thumbnail'])

		show_id = input('Enter ID: ').lower()
		for i in response['items']:
			if i['id'] == int(show_id) and i['cat_id'] > 1:
				self.get_show_by_id(show_id)
			elif i['id'] == int(show_id) and i['cat_id'] == 1:
				self.get_movie_by_id(show_id)


	def get_movie_by_id(self, movie_id: str):
		"""
    Downloads the movie with the given ID.

    Args:
        movie_id (str): The ID of the movie to download.
    """  
		movie_json = get_json(self.session, GET_VIDEO_URL.format(self.user_id, self.auth_token, movie_id))
		if self.info:
			logging.info(f"Name: {movie_json['title']}\n"
									 f"ID: {movie_json['id']}\n"
									 f"Year: {movie_json['year']}\n"
									 f"Description: {movie_json['plot']}")

		logging.info(f'[*] Downloading {movie_json["title"]} ({movie_json["year"]})')
		self.dl_obj = dl_wrapper(self.get_quality(movie_json), self.download_dir)
		if self.dl_obj:
			self.dl_obj.start()


	def get_show_by_id(self, show_id: str, season_chosen=None):
		"""
    Downloads the TV show with the given ID and prompts the user to select a season to download.

    Args:
        show_id (str): The ID of the TV show to download.
        season_chosen (str): The season to download. If not provided, the user will be prompted to select one.
    """
		season_json = get_json(self.session, GET_SEASON_URL.format(self.user_id, self.auth_token, show_id))
		season_title = season_json["show"]["title"]
		season_id = season_json['show']['id']
		logging.info(f'[*] Showing info for {season_title} (id: {season_id})')
		self.download_dir = self.download_dir + season_title.replace(' ', '_') + '/'
		if self.info:
			logging.info(f"Name: {season_json['show']['title']}\n"
										f"ID: {season_json['show']['id']}\n"
										f"Year: {season_json['show']['year']}\n"
										f"Description: {season_json['show']['plot']}\n")
		if not season_chosen:
			season_chosen = input(
				f'[?] Which season (out of {list(season_json["season_list"].keys())[0]}) would you like to download? ')

		num_episodes = len(season_json['season_list'][str(season_chosen)])

		for i in range(num_episodes):
			episode = str(season_json['season_list'][str(season_chosen)][i][1])
			self.get_single_episode(show_id, season_chosen, episode, self.download_dir)


	def get_single_episode(self, show_id: str, season: str, episode: str, path: str):
		"""
    Downloads a single episode of the given TV show.

    Args:
        show_id (str): The ID of the TV show to download.
        season (str): The season number of the episode.
        episode (str): The episode number of the episode.
        path (str): The directory where the downloaded file will be saved.
    """
		episode_info = get_json(self.session, GET_SINGLE_EPISODE_URL.format(self.user_id, self.auth_token, show_id, season, episode))
		self.dl_obj = dl_wrapper(self.get_quality(episode_info), path)
		if self.dl_obj:
			self.dl_obj.start()


	def signal_handler(self, sig, frame):
		"""
    Handles keyboard interrupts during file downloads and removes incomplete downloads.

    Args:
        sig: The signal number.
        frame: The interrupted stack frame.
    """
		logging.debug('SIGINT captured')
		if self.dl_obj:
			self.dl_obj.stop()
		for file in os.listdir(self.download_dir):
			if file.endswith(".00", -4, -1):
				'''
				  Smart_DL downloads file in chunks ending in ".00x" 
					where x is a number corresponding to the thread that is being used
				'''
				filepath = os.path.join(self.download_dir, file)
				os.remove(filepath)
		if self.download_dir and len(os.listdir(self.download_dir)) == 0 and self.download_dir is not DOWNLOAD_DIRECTORY:
			'''
			remove download dir if not the default directory
			'''
			logging.debug(f'Removing {self.download_dir}')
			os.rmdir(self.download_dir)
		logging.error('\n[!] CTRL-C pressed - exiting!')
		exit(1)
		


import ascii_magic


def image_to_ascii(image_url: str):
    """
    Convert an image from the given URL to ASCII art and display it in the terminal.

    Args:
        image_url (str): The URL of the image to convert.

    Returns:
        None: The ASCII art is displayed in the terminal.

    Raises:
        AsciiMagicError: If an error occurs while converting the image.

    Example:
        >>> image_to_ascii("https://example.com/image.jpg")

    """
    img = ascii_magic.from_url(image_url, columns=30)
    ascii_magic.to_terminal(img)



import argparse
import logging


def options_parser(): 
	# groups for general options and media options
	parser = argparse.ArgumentParser(
		description='Mobilevids Downloader script', prog='mobilevids_dl')

	parser.add_argument('search', nargs='?')
	parser.add_argument(
		'-a', 
		'--ascii', 
		help='show ascii art', 
		action='store_true')

	parser.add_argument(
		'-d', 
		'--debug', 
		help='debugs the program - duh', 
		action='store_true')

	parser.add_argument(
		'-i', 
		'--info', 
		help='show info about movie/show', 
		action='store_true')
	
	parser.add_argument(
		'-e', 
		'--episode', 
		help='download a single episode (must be used with -t [TV ID] and -s [SEASON]', 
		default=False)

	parser.add_argument(
		'-m', 
		'--movie', 
		help='downloads the ID of a movie', 
		default=False)
	
	parser.add_argument(
		'-n',
		'--netrc',
		help='use netrc for reading passwords, uses default'
        ' location if no path specified',
		default=False
	)
	
	parser.add_argument(
		'-p',
		'--password',
		help='provide a mobilevids password',
		default=False
	)

	parser.add_argument(
		'-s', 
		'--season', 
		help='specify season to download (must use with -t)', 
		default=False)

	parser.add_argument(
		'-t', 
		'--tv', 
		help='download a TV show based on it\'s ID', 
		default=False)
	
	parser.add_argument(
		'-u',
		'--username',
		help='provide a Mobilevids username',
		default=False
	)

	args = parser.parse_args()

	if args.debug:
		logging.basicConfig(level=logging.DEBUG,
			force=True,
			format='%(name)s[%(funcName)s] %(message)s')
	else:
		logging.basicConfig(level=logging.INFO,	
		force=True,
		format='%(message)s')
	
	logging.debug(f'Program arguments {args}')
	return args



# -*- coding: utf-8 -*-
#
# you can install this to a local test virtualenv like so:
#   virtualenv venv
#   ./venv/bin/pip install --editable .
#   ./venv/bin/pip install --editable .[dev]  # with dev requirements, too

from __future__ import print_function

import os.path
import subprocess
import sys
# For compatibility with Python2.7
from io import open

from setuptools import setup

from mobilevids import __VERSION__


def generate_readme_rst():
    """
    Generate README.rst from README.md via pandoc.

    In case of errors, we show a message having the error that we got and
    exit the program.
    """

    pandoc_cmd = [
        'pandoc',
        '--from=markdown',
        '--to=rst',
        '--output=README.rst',
        'README.md'
    ]

    if os.path.exists('README.rst'):
        return
    try:
        subprocess.call(pandoc_cmd)
    except (IOError, OSError) as e:
        print('Could not run "pandoc". Error: %s' % e, file=sys.stderr)
        print('Generating only a stub instead of the real documentation.')


def read_file(filename, alt=None):
    """
    Read the contents of filename or give an alternative result instead.
    """
    lines = None

    try:
        with open(filename, encoding='utf-8') as f:
            lines = f.read()
    except IOError:
        lines = [] if alt is None else alt
    return lines


generate_readme_rst()

long_description = read_file(
    'README.md',
    'Cannot read README.md'
)
requirements = read_file('requirements.txt')
dev_requirements = read_file('requirements-dev.txt')

trove_classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: End Users/Desktop',
    'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: Implementation :: CPython',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Programming Language :: Python',
    'Topic :: Education',
]

setup(
    name='mobilevids-dl',
    version=__VERSION__,
    maintainer='Aharon Maslin',
    maintainer_email='aronmas613@gmail.com',

    license='LGPL',
    url='https://github.com/ahron-maslin/mobilevids-dl',

    install_requires=requirements,
    extras_require=dict(
        dev=dev_requirements
    ),

    description='Script for downloading Movies and Shows from mobilevids.org',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=['mpbilevids-dl', 'mobilevids',
              'download', 'entertainment', 'video'],
    classifiers=trove_classifiers,

    packages=["mobilevids"],
    entry_points=dict(
        console_scripts=[
            'mobilevids-dl=mobilevids.dispatcher:main'
        ]
    ),

    platforms=['any'],
)



import requests
import netrc
import json
from json import JSONDecodeError
from wget import detect_filename
import os
import logging
from pySmartDL import SmartDL

from .define import LOGIN_PAYLOAD, LOGIN_URL, HEADERS, AUTH_TOKEN_CACHE, GET_VIDEO_URL, NETRC_FILE_PATH


def check_auth_cache(cached_auth_file):
	"""
	Check if cached auth file exists, and return its content as a string.

	Args:
		cached_auth_file: The path to the cached auth file.

	Returns:
		The content of the cached auth file as a string.
	"""
	if os.path.isfile(cached_auth_file):
		with open(cached_auth_file, "r") as auth:
			return auth.read()


def save_cached_creds(login_info: dict):
	"""
	Write the given login_info to the cached auth file.

	Args:
		login_info: A dictionary containing login information.
	"""
	with open(AUTH_TOKEN_CACHE, "w") as cache_file:
		cache_file.write(json.dumps(login_info))



def session_init():
	"""
	Initialize a new requests session.

	Returns:
		A new requests session object.
	"""
	session = requests.Session()
	logging.debug('Created Session')
	return session


def get_creds(session, username=None, password=None) -> tuple:
	"""
	Attempt to log in to the MobileVids website using either cached credentials or
	the login credentials stored in the netrc file.

	Args:
		session: A requests session object.

	Returns:
		A tuple containing the authentication token and the user ID, in that order.
	"""
	# try using the cached credentials
	if os.path.isfile(AUTH_TOKEN_CACHE):
		with open(AUTH_TOKEN_CACHE, "r") as cache_file:
			cached_creds = json.load(cache_file)

		url = GET_VIDEO_URL.format(cached_creds['id'], cached_creds['auth_token'], "1")

		creds = get_json(session, url)

		if creds['status'] != "-1":
			logging.info("[*] Using cached credentials")
			return cached_creds['auth_token'], cached_creds['id']

	if username and password:
		logging.debug(f'logging in with username: {username} and password: {password}')
		pass
	else:
		try:
			creds = netrc.netrc(NETRC_FILE_PATH).authenticators('mobilevids')
			username, password = creds[0], creds[2]
			logging.debug('Trying netrc file %s', os.path)
		except (IOError, netrc.NetrcParseError):
			raise BaseException(
			'''
			Did not find valid netrc file: 
			Create a .netrc file in the package directory with the following format:
				machine = mobilevids
				login = MOBILEVIDS_USERNAME
				password = MOBILEVIDS_PASSWORD
			'''
			)
		
	login_string = LOGIN_PAYLOAD.format(username=username, password=password)

	try:
		login_info = json.loads(session.post(
			LOGIN_URL, data=login_string, headers=HEADERS).text)
	except JSONDecodeError:
		raise JSONDecodeError('cannot decode JSON - bad response!')
	save_cached_creds(login_info)
	logging.debug(login_info)
	logging.info('[*] Successfully logged in!')
	return login_info['auth_token'], login_info['id']


def dl_wrapper(video: str, folder):  
	"""
	Wrapper function for the SmartDL module.

	Args:
		video: The URL of the video to download.
		folder: The path to the folder in which to save the downloaded video.

	Returns:
		A SmartDL object that can be used to download the video.
	"""
	if not os.path.exists(folder): # remove this? smart_DL takes care of it
		os.mkdir(folder)

	filename = detect_filename(video)
	save_path = folder + filename

	if not os.path.isfile(save_path):
		logging.debug(f'Save path {save_path}')
		logging.info(f'Downloading {filename} to {folder}')
		dl_obj = SmartDL(video, save_path, threads=(os.cpu_count()-2))
		return dl_obj
	
	return None


def get_json(session, url: str) -> dict:
	"""
	Sends a GET request to the given URL using the provided session and returns the JSON response as a dictionary.

	Args:
		session: The session to use for the request.
		url: The URL to send the request to.

	Returns:
		A dictionary containing the JSON response.
"""
	response = session.get(url)
	response_json = json.loads(response.text)
	logging.debug(response.headers)
	logging.debug(response_json)
	
	return response_json
